<?php

/**
 * @file
 * Timepad Module API File.
 */

namespace Drupal\timepad\Api;

use Drupal\Core\Config\ConfigFactoryInterface;
use GuzzleHttp\Client;

/**
 * Class TimePadApi.
 */
class TimePadApi implements TimePadApiInterface {

  /**
   * URL for API.
   */
  const URL = 'https://api.timepad.ru/';

  /**
   * Api version.
   */
  const VERSION = 'v1';

  /**
   * Auth URL for API.
   */
  const AUTH_URL = 'https://api.timepad.ru/oauth/authorize';

  /**
   * HTTP method GET.
   */
  const METHOD_GET = 'GET';

  /**
   * HTTP method POST.
   */
  const METHOD_POST = 'POST';

  /**
   * HTTP client.
   *
   * @var \GuzzleHttp\Client
   */
  protected $client;

  /**
   * TimePad config.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  private $config;

  /**
   * Authorize token, for protected API methods.
   *
   * @var string
   */
  protected $token;

  /**
   * Class constructor.
   *
   * @param Client $client
   * @param ConfigFactoryInterface $configFactory
   */
  public function __construct(Client $client, ConfigFactoryInterface $configFactory) {
    $this->client = $client;
    $this->config = $configFactory->get('timepad.settings');
    $this->token = $this->config->get('access_token');
  }

  /**
   * Generate url for request.
   *
   * @param string $path
   *   Protected function getUrl $path.
   * @param string $base
   *   Protected function getUrl $base.
   *
   * @return string
   *   Protected function getUrl string.
   */
  protected function getUrl($path, $base = '') {
    $base = $base ? $base : self::URL;
    return $base . $path;
  }

  /**
   * {@inheritDoc}
   */
  public function eventsGetList(array $params = array()) {
    try {
      $result = $this->entityAction($params, static::METHOD_GET, 'events');
    }
    catch (\Exception $e) {
      if ($e->getCode() == 404) {
        // Event not found.
        $result = FALSE;
      }
      else {
        throw $e;
      }
    }

    return $result;
  }

  /**
   * {@inheritDoc}
   */
  public function eventsCreate(array $params) {
    return $this->entityAction($params, static::METHOD_POST, 'events');
  }

  /**
   * {@inheritDoc}
   */
  public function eventsEdit($event_id, array $params) {
    return $this->entityAction($params, static::METHOD_POST, 'events', $event_id);
  }

  /**
   * {@inheritDoc}
   */
  public function eventsGet($event_id, array $params = array()) {
    try {
      $result = $this->entityAction($params, static::METHOD_GET, 'events', $event_id);
    }
    catch (\Exception $e) {
      if ($e->getCode() == 404) {
        // Event not found.
        $result = FALSE;
      }
      else {
        throw $e;
      }
    }

    return $result;
  }

  /**
   * {@inheritDoc}
   */
  public function organizationsCreate(array $params) {
    return $this->entityAction($params, static::METHOD_POST, 'organizations');
  }

  /**
   * Makes an action (Create / Save/ Delete) with any timepad entity.
   *
   * @param array $params
   *   Array with params to operate.
   * @param string $method
   *   Private function entityAction string method.
   *
   * @return array
   *
   * @throws \Exception
   */
  private function entityAction(array $params, $method) {
    // Get args to format URL.
    $urlParts = func_get_args();
    // First argument will be "params", we don't need it this way.
    $urlParts[0] = static::VERSION;
    // Remove $method argument.
    unset($urlParts[1]);

    // Format URL form all other arguments.
    $url = static::URL . implode('/', $urlParts);

    return $this->request(
      $url,
      $method,
      $params,
      array(
        'Authorization: Bearer ' . $this->token,
      )
    );
  }

  /**
   * {@inheritDoc}
   */
  public function accountGet() {
    $params = array(
      'token' => $this->token,
    );

    $url = static::URL . 'introspect';
    $result = $this->request($url, static::METHOD_POST, $params);

    return $result;
  }

  /**
   * Execution of the request.
   *
   * @param string $url
   *   url.
   * @param string $method
   *   string method.
   * @param array $parameters
   *   string parameters.
   * @param array $headers
   *   array headers.
   *
   * @return array
   *   array.
   *
   * @access protected
   *
   * @throws \Exception
   * @throws \GuzzleHttp\Exception\GuzzleException
   */
  protected function request($url, $method = self::METHOD_GET, array $parameters = [], array $headers = []) {
    $options = [
      'headers' => $headers,
    ];

    if (!empty($parameters)) {
      if ($method == static::METHOD_GET) {
        $url .= '?' . http_build_query($parameters);
      }
      else {
        $options['json'] = $parameters;
      }
    }

    // Get curl handler or initiate it.
    $response = $this->client->request($method, $url, $options);
    $result = json_decode($response->getBody(), TRUE);
    $statusCode = $response->getStatusCode();

    if ($statusCode >= 400) {
      if (!empty($result['response_status']['errors'])) {
        foreach ($result['response_status']['errors'] as $error) {
          $message = '';
          if (!empty($error['error_code'])) {
            $message .= 'Error ' . $error['error_code'] . ': ';
          }
          if (!empty($error['message'])) {
            $message .= $error['message'] . ' ';
          }
          if (!empty($error['field_name'])) {
            $message .= $error['field_name'];
          }
          if (!empty($message)) {
            \Drupal::messenger()->addMessage($message, 'error');
          }
        }
      }
      throw new \Exception($result['response_status']['message'], $statusCode);
    }

    return !empty($result) ? $result : [];
  }

  /**
   * {@inheritDoc}
   */
  public function ordersCreate($event_id, array $params) {
    return $this->entityAction($params, static::METHOD_POST, 'events', $event_id, 'orders');
  }

  /**
   * {@inheritDoc}
   */
  public function ordersGetList($event_id, array $params = array()) {
    try {
      $result = $this->entityAction($params, static::METHOD_GET, 'events', $event_id, 'orders');
    }
    catch (\Exception $e) {
      if ($e->getCode() == 404) {
        // Event not found.
        $result = FALSE;
      }
      else {
        throw $e;
      }
    }

    return $result;
  }

  /**
   * {@inheritDoc}
   */
  public function ordersGet($order_id, $event_id, array $params = array()) {
    try {
      $result = $this->entityAction($params, static::METHOD_GET, 'events', $event_id, 'orders', $order_id);
    }
    catch (\Exception $e) {
      if ($e->getCode() == 404) {
        // Event not found.
        $result = FALSE;
      }
      else {
        throw $e;
      }
    }

    return $result;
  }

  /**
   * {@inheritDoc}
   */
  public function webhooksGetList($organization_id, array $params = array()) {
    try {
      $result = $this->entityAction($params, static::METHOD_GET, 'organizations', $organization_id, 'hooks');
    }
    catch (\Exception $e) {
      if ($e->getCode() == 404) {
        // Event not found.
        $result = FALSE;
      }
      else {
        throw $e;
      }
    }

    return $result;
  }

  /**
   * {@inheritDoc}
   */
  public function webhooksCreate($organization_id, array $params) {
    return $this->entityAction($params, static::METHOD_POST, 'organizations', $organization_id, 'hooks');
  }

  /**
   * {@inheritDoc}
   */
  public function webhooksGet($hook_id, $organization_id, array $params = array()) {
    try {
      $result = $this->entityAction($params, static::METHOD_GET, 'organizations', $organization_id, 'hooks', $hook_id);
    }
    catch (\Exception $e) {
      if ($e->getCode() == 404) {
        // Event not found.
        $result = FALSE;
      }
      else {
        throw $e;
      }
    }

    return $result;
  }

  /**
   * {@inheritDoc}
   */
  public function webhooksEdit($hook_id, $organization_id, array $params) {
    return $this->entityAction($params, static::METHOD_POST, 'organizations', $organization_id, 'hooks', $hook_id);
  }

}
